using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Infragistics.Win.UltraWinSchedule;
using Infragistics.Win;

namespace WinGanttView_CustomColumns
{
    public partial class frmMain : Form
    {
        #region Properties
        // Property used to hide the GDI+ drawing when a user interacts with the sample
        Boolean drawGDI = true;
        #endregion

        #region Initialize GanttView (Includes Custom Column Code)
        private void InitGanttView()
        {
            // Create a new Project other than the Unassigned Project
            Project shippingProject = calendarInfoShipping.Projects.Add(Properties.Resources.ProjectName, DateTime.Today);
            shippingProject.Key = "ShippingProject";

            // Add custom columns
            calendarInfoShipping.CustomTaskColumns.Add(Properties.Resources.ColumnOneName, typeof(String), false, String.Format("{0} 123", Properties.Resources.TruckName));
            calendarInfoShipping.CustomTaskColumns.Add(Properties.Resources.ColumnTwoName, typeof(String), false, "13000");
            calendarInfoShipping.CustomTaskColumns.Add(Properties.Resources.ColumnThreeName, typeof(String), false, String.Format("{0} 123", Properties.Resources.DoorName));

            // Hide unnecessary columns
            ganttViewShipping.GridSettings.ColumnSettings[TaskField.Duration].Visible = Infragistics.Win.DefaultableBoolean.False;
            ganttViewShipping.GridSettings.ColumnSettings[TaskField.Resources].Visible = Infragistics.Win.DefaultableBoolean.False;
            ganttViewShipping.GridSettings.ColumnSettings[TaskField.Dependencies].Visible = Infragistics.Win.DefaultableBoolean.False;

            // Set Start and Finish columns to use short date format
            ganttViewShipping.GridSettings.ColumnSettings[TaskField.StartDateTime].Format = Properties.Resources.DateStringFormat;
            ganttViewShipping.GridSettings.ColumnSettings[TaskField.EndDateTime].Format = Properties.Resources.DateStringFormat;

            // Create a Summary or Parent Task
            DateTime StartTime = DateTime.Parse(String.Format("{0} 8:00 AM", DateTime.Now.ToShortDateString()));
            Task truckTask = calendarInfoShipping.Tasks.Add(StartTime, TimeSpan.FromDays(3.75), DateTime.Today.DayOfWeek.ToString(), "ShippingProject");
            
            // Create child tasks
            Task deliveryTask143 = truckTask.Tasks.Add(StartTime, TimeSpan.FromDays(2.3), String.Format("{0} #143", Properties.Resources.StoreName));
            Task deliveryTask276 = truckTask.Tasks.Add(StartTime, TimeSpan.FromDays(1.1), String.Format("{0} #276", Properties.Resources.StoreName));
            Task deliveryTask073 = truckTask.Tasks.Add(StartTime, TimeSpan.FromDays(0.24), String.Format("{0} #073", Properties.Resources.StoreName));

            // Bind calendarInfoShipping with the WinGanttView
            ganttViewShipping.CalendarInfo = calendarInfoShipping;
            ganttViewShipping.Project = ganttViewShipping.CalendarInfo.Projects[1];

            // Add the end time for the delivery to the chart area
            ganttViewShipping.TimelineSettings.BarSettings.BarTextRight = Infragistics.Win.UltraWinSchedule.TaskUI.BarTextField.EndDateTime;
            ganttViewShipping.TimelineSettings.BarSettings.BarTextDateTimeFormat = "T";

            // Resize all columns to fit their text
            ganttViewShipping.PerformAutoSizeAllGridColumns();

            // Adjust the splitter
            ganttViewShipping.GridAreaWidth = 470;
        }
        #endregion

        #region Form Constructor
        public frmMain()
        {
            InitializeComponent();
        }
        #endregion

        #region Events
        private void frmMain_Load(object sender, EventArgs e)
        {
            InitGanttView();
            ganttViewShipping.Paint += new PaintEventHandler(ganttViewShipping_Paint);
            ganttViewShipping.PropertyChanged += new Infragistics.Win.PropertyChangedEventHandler(ganttViewShipping_PropertyChanged);
        }

        void ganttViewShipping_PropertyChanged(object sender, Infragistics.Win.PropertyChangedEventArgs e)
        {
            drawGDI = false;
        }

        private void ganttViewShipping_InitializeTaskGridRow(object sender, Infragistics.Win.UltraWinGanttView.InitializeTaskGridRowEventArgs e)
        {
            // Clear out the custom column values for the first task row
            if (e.Task.RowNumber == 1)
            {
                e.Task.SetCustomProperty(Properties.Resources.ColumnOneName, String.Empty);
                e.Task.SetCustomProperty(Properties.Resources.ColumnTwoName, String.Empty);
                e.Task.SetCustomProperty(Properties.Resources.ColumnThreeName, String.Empty);
                return;
            }

            // Randomizing the custom column values
            e.Task.SetCustomProperty(Properties.Resources.ColumnOneName, String.Format("{0} {1}",
                Properties.Resources.TruckName,
                new Random(e.Task.RowNumber * DateTime.Now.Millisecond).Next(999).ToString()));
            e.Task.SetCustomProperty(Properties.Resources.ColumnTwoName,
                new Random(e.Task.RowNumber * DateTime.Now.Millisecond).Next(13000).ToString());
            e.Task.SetCustomProperty(Properties.Resources.ColumnThreeName, String.Format("{0} {1}",
                Properties.Resources.DoorName,
                new Random(e.Task.RowNumber * 2 * DateTime.Now.Millisecond).Next(50).ToString()));

            // Format the cell appearance based on value
            if (int.Parse(e.Task.GetCustomProperty(Properties.Resources.ColumnTwoName).ToString()) > 7000)
                e.Task.GridSettings.CellSettings[Properties.Resources.ColumnTwoName].Appearance.BackColor = Color.LightSalmon;
            else
                e.Task.GridSettings.CellSettings[Properties.Resources.ColumnTwoName].Appearance.BackColor = Color.LightGreen;
        }
        #endregion

        #region Custom GDI+
        void ganttViewShipping_Paint(object sender, PaintEventArgs e)
        {
            if (!drawGDI)
                return;

            Infragistics.Win.UltraWinGanttView.UIElements.GridAreaUIElement uie = ganttViewShipping.UIElement.ChildElements[0] as Infragistics.Win.UltraWinGanttView.UIElements.GridAreaUIElement;

            //GridAreaUIElement > DataAreaUIElement > RowColRegionIntersectionUIElement
            UIElement inner_uie = uie.ChildElements[0].ChildElements[0];
            foreach (UIElement element in inner_uie.ChildElements)
            {
                if (element is Infragistics.Win.UltraWinGrid.BandHeadersUIElement)
                {
                    foreach (UIElement headerElement in element.ChildElements)
                    {
                        if (headerElement is Infragistics.Win.UltraWinGrid.HeaderUIElement)
                        {
                            Infragistics.Win.UltraWinGrid.HeaderUIElement header = headerElement as Infragistics.Win.UltraWinGrid.HeaderUIElement;

                            if (header.Header.Caption == Infragistics.Win.UltraWinGanttView.Resources.GetString("TaskProxy_PropertyDisplayName_Task.EndDateTime"))
                            {
                                Pen linePen = new Pen(Brushes.Red, 4);
                                Pen arrowPen = new Pen(Brushes.DarkBlue, 30);
                                int pX = headerElement.Rect.Right, pY = headerElement.Rect.Top;
                                
                                e.Graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;

                                e.Graphics.DrawLine(linePen, pX, pY, pX, pY + 115);

                                arrowPen.StartCap = System.Drawing.Drawing2D.LineCap.ArrowAnchor;
                                e.Graphics.DrawLine(arrowPen, 150, 120, 150, 180);
                                e.Graphics.DrawString(Properties.Resources.GDI_BuiltIn, 
                                    new Font(FontFamily.GenericSansSerif, 14), Brushes.Black, 80, 185);
                                e.Graphics.DrawLine(arrowPen, 385, 120, 385, 180);
                                e.Graphics.DrawString(Properties.Resources.GDI_CustomColumns, 
                                    new Font(FontFamily.GenericSansSerif, 14), Brushes.Black, 315, 185);
                                linePen.Dispose();
                                arrowPen.Dispose();
                                break;
                            }
                        }
                    }
                }
            }
        }
        #endregion
    }
}